/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
* This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.checklist.CheckList;
import org.greenstone.gatherer.checklist.Entry;
import org.greenstone.gatherer.collection.CollectionConfiguration;
import org.greenstone.gatherer.util.Utility;

public class SuperCollectionManager {

    static final public String SUPERCOLLECTION_COMMAND = "supercollection";
    static final public String CCS_COMMAND = "ccs";

    private String current_coll_name = null;
    private Control controls;
    private ArrayList collection_checklist_model;

    SuperCollectionManager() {
	// We start by building a model of the installed collections.
	collection_checklist_model = new ArrayList();
	File gsdl_collection_directory = new File(Utility.getCollectionDir(Gatherer.config.gsdl_path));
	current_coll_name = Gatherer.c_man.getCollection().getName();
	File[] possible_collections = gsdl_collection_directory.listFiles();
	for(int i = 0; possible_collections != null && i < possible_collections.length; i++) {
	    if (possible_collections[i].getName().equals(current_coll_name)) {
		// we dont want to add the current collection to the list
		continue;
	    }
	    // The simpliest case is if the directory etc/collect.cfg file and a metadata/ in it. Thus it can easily be built upon.
	    File collect_cfg_file = new File(possible_collections[i], Utility.CONFIG_DIR);
	    if(collect_cfg_file.exists()) {
		CollectionConfiguration collect_cfg = new CollectionConfiguration(collect_cfg_file);
		String collection_title = collect_cfg.getName();
		String collection_name = possible_collections[i].getName();
		// We do have to block the model collection.
		if(!collection_title.equals("**title**")) {
		    Entry entry = new Entry(collection_title);
		    entry.setProperty(collection_name);
		    collection_checklist_model.add(entry);
		    entry = null;
		}
		collection_name = null;
		collection_title = null;
		collect_cfg = null;
	    }
	    collect_cfg_file = null;
	}
	possible_collections = null;
	gsdl_collection_directory = null;
	// Sort the result.
	Collections.sort(collection_checklist_model);
    }

    public boolean parse(String command) {
	String command_lc = command.toLowerCase();
	if(command_lc.startsWith(SUPERCOLLECTION_COMMAND) || command_lc.startsWith(CCS_COMMAND)) {
	    StringTokenizer tokenizer = new StringTokenizer(command);
	    tokenizer.nextToken();
	    // Read in each of the collection names.
	    ArrayList collection_names = new ArrayList();
	    while(tokenizer.hasMoreTokens()) {
		collection_names.add(tokenizer.nextToken());
	    }
	    // Now that we have the collection names, select any entries in the checklist that match.
	    for(int i = 0; i < collection_checklist_model.size(); i++) {
		Entry entry = (Entry) collection_checklist_model.get(i);
		String collection_name = entry.getProperty();
		if(collection_names.contains(collection_name)) {
		    entry.setSelected(true);
		    collection_names.remove(collection_name); // Makes successive runs faster.
		}
		collection_name = null;
		entry = null;
	    }
	    // Done.
	    return true;
	}
	return false;
    }

    /** Method to retrieve the control for this manager.
     * @return A JPanel containing the controls.
     */
    public JPanel getControls() {
	if(controls == null) {
	    controls = new Control();
	}
	return controls;
    }

    public void invalidateControls() {
	controls = null;
    }

    /** Return the contents of this manager as a command or block of commands ready to be written to the collection configuration file. 
     * @return A String containing a configuration command or block of commands, terminated with a new line. This value will be null if there was no supercollection command set. */
    public String toString() {
	StringBuffer command = new StringBuffer(SUPERCOLLECTION_COMMAND);
	// Now for each entry selected output its collection name (property setting)
	boolean found_entry = false;
	for(int i = 0; i < collection_checklist_model.size(); i++) {
	    Entry entry = (Entry) collection_checklist_model.get(i);
	    if(entry.isSelected()) {
		found_entry = true;
		command.append(" ");
		command.append(entry.getProperty());
	    }
	    entry = null;
	}
	// It is possible that there were no entries in which case we return null.
	if(found_entry) {
	    // we need to add on the current collection
	    command.append(" ");
	    command.append(current_coll_name);
	    command.append("\n");
	    command.append("\n");
	    return command.toString();
	}
	else {
	    return null;
	}
    }

    /** Call dictionary get with just a key.
     * @param key A String which is mapped to a initial String within the ResourceBundle.
     * @return A String which has been referenced by the key String and that either contains no argument fields, or has had the argument fields automatiically populated with formatting Strings of with argument String provided in the get call.
     */
    private String get(String key) {
	if(key.indexOf(".") == -1) {
	    key = "CDM.SuperCollectionManager." + key;
	}
	return Gatherer.dictionary.get(key);
    }

    /** Provides controls for altering the SuperCollection settings. */
    private class Control
	extends JPanel {

	private boolean init = true;
	//private JSplitPane center_pane;

	Control() {
	    super();
	    // Creation
	    JPanel header_panel = new JPanel();
	    
	    JLabel title_label = new JLabel(get("Title"));
	    title_label.setHorizontalAlignment(JLabel.CENTER);
	    title_label.setOpaque(true);

	    //center_pane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);

	    JTextArea instructions = new JTextArea(get("Instructions"));
	    instructions.setBackground(Gatherer.config.getColor("coloring.collection_tree_background", false));
	    instructions.setCaretPosition(0);
	    instructions.setEditable(false);
	    instructions.setLineWrap(true);
	    instructions.setWrapStyleWord(true);

	    CheckList collection_checklist = new CheckList(collection_checklist_model);
	    // Layout
	    //center_pane.setTopComponent(new JScrollPane(instructions));
	    //center_pane.setBottomComponent(new JScrollPane(collection_checklist));
	    header_panel.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	    header_panel.setLayout(new BorderLayout());
	    header_panel.add(title_label, BorderLayout.NORTH);
	    header_panel.add(new JScrollPane(instructions), BorderLayout.CENTER);
	    
	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(header_panel, BorderLayout.NORTH);
	    add(new JScrollPane(collection_checklist), BorderLayout.CENTER);
	}

	public void paint(Graphics g) {
	    super.paint(g);
	    // If this is the first time we've painted this control, then set the divider 1/3:2/3
	    //if(init) {
	    //	init = false;
	    //	center_pane.setDividerLocation(0.3);
	    //	super.paint(g);
	    //}
	}
    }
}
