package org.greenstone.gsdl3.core;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

// other java classes
import java.io.File;
import java.util.HashMap;
import java.util.Enumeration;

/** A receptionist that uses xslt to transform the page_data before returning it. . Receives requests consisting
 * of an xml representation of cgi args, and returns the page of data - in 
 * html by default. The requests are processed by the appropriate action class
 *
 * @see Action
 */
public class TransformingReceptionist extends Receptionist{
    
    /** the list of xslt to use for actions */
    protected HashMap xslt_map = null;
    
    /** a transformer class to transform xml using xslt */
    protected XMLTransformer transformer=null;

    public TransformingReceptionist() {
	super();
	this.xslt_map = new HashMap();
	this.transformer = new XMLTransformer();
    }
    
    /** configures the receptionist - overwrite this to set up the xslt map*/
    public boolean configure() {
	
	if (this.config==null) {
	    System.err.println("Receptionist Error: config variables must be set before calling configure");
	    return false;
	}
	if (this.mr==null) {	   
	    System.err.println("Receptionist Error: message router must be set  before calling configure");
	    return false;
	}
	
	// find the config file containing a list of actions
	File interface_config_file = new File(GSFile.interfaceConfigFile(GSFile.interfaceHome(this.config.gsdl3_home_, this.config.interface_name_)));
	if (!interface_config_file.exists()) {
	    System.err.println("TransformingReceptionist: interface config file: "+interface_config_file.getPath()+" not found!");
	    return false;
	}
	
	Element config_doc = this.converter.getDOM(interface_config_file).getDocumentElement();
	Element action_list = (Element)GSXML.getChildByTagName(config_doc, GSXML.ACTION_ELEM+GSXML.LIST_MODIFIER);
	NodeList actions = action_list.getElementsByTagName(GSXML.ACTION_ELEM);

	for (int i=0; i<actions.getLength(); i++) {
	    Element action = (Element) actions.item(i);
	    String class_name = action.getAttribute("class");
	    String action_name = action.getAttribute("name");
	    Action ac = null;
	    try {
		ac = (Action)Class.forName("org.greenstone.gsdl3.action."+class_name).newInstance();
	    } catch (Exception e) {
		System.out.println("couldn't load in action "+class_name);
		e.printStackTrace();
		continue;
	    }
	    ac.setConfigVars(this.config);
	    ac.setMessageRouter(this.mr);
	    ac.configure();
	    this.action_map.put(action_name, ac);

	    // now do the xslt map
	    String xslt = action.getAttribute("xslt");
	    if (!xslt.equals("")) {
		this.xslt_map.put(action_name, xslt);
	    }
	    NodeList subactions = action.getElementsByTagName(GSXML.SUBACTION_ELEM);
	    for (int j=0; j<subactions.getLength(); j++) {
		Element subaction = (Element)subactions.item(j);
		String subname = subaction.getAttribute(GSXML.NAME_ATT);
		String subxslt = subaction.getAttribute("xslt");
		
		String map_key = action_name+":"+subname;
		System.out.println("adding in to xslt map, "+map_key+"->"+subxslt);
		this.xslt_map.put(map_key, subxslt);
	    }
	    
	   
	}

	return true;
    }


    protected Element postProcessPage(Element page) {
	
	// might need to add some data to the page
	addExtraInfo(page);
	// transform the page using xslt
	Element transformed_page = transformPage(page);
	
	return transformed_page;
    }
    
    /** adds in the config stuff and interface display stuff - 
     * needed by the xslt
     */
    protected void addExtraInfo(Element page) {
	Element page_extra = this.doc.createElement(GSXML.PAGE_EXTRA_ELEM);
	page.appendChild(page_extra);
	String lang = ((Element)GSXML.getChildByTagName(page, GSXML.PAGE_REQUEST_ELEM)).getAttribute(GSXML.LANG_ATT);
	Element display = getDisplayElement(lang);
	page_extra.appendChild(this.doc.importNode(display, true));
	page_extra.appendChild(this.doc.importNode(this.config.config_xml_, true));
	
	return;
    }

    /** transform teh page using xslt
     * we need to get any format element out of teh page and add it to the xslt
     * before transforming */
    protected Element transformPage(Element page) {

	Element request = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_REQUEST_ELEM);
	String action = request.getAttribute("action");
	String subaction = request.getAttribute("subaction");
	String xslt_file = getXSLTFileName(action, subaction);
	if (xslt_file==null) {
	    // cant transform the page - return null or the original?
	    System.err.println(" cant find the xslt file needed, so returning the original page!");
	    return page;
	}
	Document style_doc = this.converter.getDOM(new File(xslt_file));

	// look for the format element in the page response
	Element page_response = (Element)GSXML.getChildByTagName(page, GSXML.PAGE_RESPONSE_ELEM);
	Element format_elem = (Element)GSXML.getChildByTagName(page_response, GSXML.FORMAT_ELEM);
	if (format_elem != null) {
	    page_response.removeChild(format_elem);
	    
	    // need to transform the format info
	    String stylesheet_file = GSFile.stylesheetFile(this.config, "config_format.xsl");
	    Document stylesheet = this.converter.getDOM(new File(stylesheet_file));
	    Element new_format = (Element)this.transformer.transform(stylesheet, format_elem);
	    // add it in to the main stylesheet
	    GSXSLT.mergeStylesheets(style_doc, new_format);
	    //System.out.println("the converted stylesheet is:");
	    //System.out.println(this.converter.getPrettyString(style_doc.getDocumentElement()));
	}
	
	GSXSLT.absoluteIncludePaths(style_doc, this.config);
	return (Element)this.transformer.transform(style_doc, page);          
    }

    protected String getXSLTFileName(String action, String subaction) {

	String name = null;
	if (!subaction.equals("")) {
	    String key = action+":"+subaction;
	    name = (String) this.xslt_map.get(key);
	}
	// try the action by itself
	if (name==null) {
	    name = (String) this.xslt_map.get(action);
	}
	// now find the absolute path
	String stylesheet = GSFile.stylesheetFile(this.config, name);
	if (stylesheet==null) {
	    System.out.println("cant find stylesheet for "+name);
	}
	return stylesheet;
    }
    /** returns an xml element containing all the text strings 
     * needed for an interface 
     * currently only uses the current interface - should add in 
     * needed ones from the default interface if current != default */
    protected Element getDisplayElement(String lang) {

	Element display = this.doc.createElement(GSXML.DISPLAY_ELEM);
	// looks for properties files based on interface name
	String resource_name = "interface_"+this.config.interface_name_;

	Dictionary dict = new Dictionary(resource_name, lang);
	Enumeration enum = dict.getKeys();
	while (enum.hasMoreElements()) {
	    String key = (String)enum.nextElement();
	    String value = dict.get(key);
	    Element e = GSXML.createTextElement(this.doc, key, value);
	    display.appendChild(e);
	}
	return display;
    }

}
