/*
 *    NzdlRequest.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

import java.util.List;

import org.nzdl.gsdl.corba.gsdlInterface.*;
import org.nzdl.gsdl.util.*;

/**
 * An object that holds CORBA request data ready for by servicing by
 * {@link NzdlService NzdlService}. NzdlRequest can receive data from: <br>
 * - a {@link NzdlQuery NzdlQuery} object <br>
 * - a document ID <br>
 * - document ID's and a metatag.
 *
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @author Aziz Mahoui    (amahoui@yahoo.com)
 * @author Gordon Paynter (paynter@cs.waikato.ac.nz)
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @author George Buchanan (g.buchanan@mdx.ac.uk)
 * @version $Revision: 2920 $
 */
public class NzdlRequest extends java.lang.Object {

  private corbaFilterRequest m_Filter = null;


    public NzdlRequest( corbaFilterRequest request ) {
	m_Filter = request;
    }

  /**
   * Creates an instance of NzdlRequest based on a search string inside a
   * {@link NzdlQuery NzdlQuery} object. When serviced by
   * {@link NzdlService NzdlService}, this type of request will return
   * references to zero or more documents. <br>
   * Pre:  Have already created instance of NzdlQuery object containing a
   * search string.<br>
   * Post: Created a query filter request that is ready for servicing.
   * @param query a {@link NzdlQuery NzdlQuery} object containing a string
   * search and options.
   */
  public NzdlRequest( NzdlQuery _query ) {
    m_Filter = NzdlCorbaFactory.createQueryFilterRequest( _query );
  }

  /**
   *  Creates an instance of NzdlRequest based on a classifier string
   *  inside a {@link NzdlBrowse NzdlBrowse} object.
   */
  public NzdlRequest( NzdlBrowse _browse) {
    m_Filter = NzdlCorbaFactory.createBrowseFilterRequest( _browse );
  }

  /**
   * Creates an instance of NzdlRequest based on a document ID. When serviced
   * by {@link NzdlService NzdlService} this type of request will return
   * exactly one document reference. <br>
   * Pre:  Have obtained a valid document ID.<br>
   * Post: Created a browse filter request
   * @param docID a string containing a valid GSDL document identifier.
   */
  public NzdlRequest( String _docID ) {
    m_Filter = NzdlCorbaFactory.createBrowseFilterRequest( _docID );
  }

  /**
   * Creates an instance of NzdlRequest based on a document ID and
   * a metatag. <br>
   * Pre:  Have obtained a valid document ID and a metatag field.<br>
   * Post: Created a metadata filter request.
   * @param docID a string containing a valid GSDL document identifier.
   * @param metatag a string containing a valid GSDL metadata such as Title
   * or Author.
   */
  public NzdlRequest( String _docID, String _metaTag ) {
    m_Filter = NzdlCorbaFactory.createMetaDataFilterRequest( _docID, _metaTag );
  }

  /**
   * Creates an instance of NzdlRequest based on a list of document ID's and
   * a metatag. <br>
   * Pre:  Have obtained a list of valid document ID's and a metatag field.<br>
   * Post: Created a metadata filter request.
   * @param docIDs a list of strings containing a valid GSDL document
   * identifiers.
   * @param metatag a string containing a valid GSDL metadata type such as
   * Title Author.
   */
  public NzdlRequest( List _docIDs, String _metaTag ) {
    m_Filter = NzdlCorbaFactory.createMetaDataFilterRequest( _docIDs, _metaTag );
  }

  /**
   * Returns the corbaFilterRequest for the NzdlRequest object.
   * @return The corbaFilterRequest Object.
   */
  public corbaFilterRequest getFilter() {
    return m_Filter;
  }
  
}
